#!/usr/bin/python

"""
units.py - Mouse-controlled units
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *

import resources
from math import *

class MouseControlableUnit(object):
    def __init__(self,id,type,charid):
        
        self.id = id
        self.charid = charid
        self.x = -1
        self.y = -1
        self.heading = 'right'
        self.type = type
        
        self.alive = True
        self.floataway = False
        self.prevvx = 0
        self.prevvy = 0
        self.gaittype = 'stand'
        self.gait = 0
        self.gait_to = 10
        
        self.speed = 4
        
        self.path = []
        self.task = []
        self.numpowerpacks = 0
        
        self.escapepod = False
        
        self.tilesize = 24
    
    def set_handles(self,handles):
        (self.h_camera, self.h_tiledlayers, self.h_msgdialogue, self.h_control) = handles
    
    def SetTile(self, tile):
        self.x = self.tilesize*(tile%self.h_tiledlayers.map_size[0]) + self.tilesize/2
        self.y = self.tilesize*(tile/self.h_tiledlayers.map_size[0]) + self.tilesize/2
    
    def Killed(self, cause):
        self.alive = False
        self.path = []
        self.task = []
        tsize = self.h_tiledlayers.tilesize
        mapw = self.h_tiledlayers.map_size[0]
        tile = mapw*(self.y/tsize)+(self.x/tsize)
        if self.id == self.h_control.unit_id:
            self.h_control.unit_id = -1 # unselect
        if cause == 'fire':
            self.h_msgdialogue.DisplayMsg("unitkilledfire")
        elif cause == 'vacuum':
            self.h_tiledlayers.RemoveObj(tile,self.id)
            self.h_msgdialogue.DisplayMsg("unitkilledvacuum")
            paths = []
            for vs in self.h_tiledlayers.vacsources:
                paths.append(self.h_planner.astar_path(tile, vs[0]))
            pathlengths = [len(p) for p in paths]
            if all([l == 0 for l in pathlengths]):
                self.path = []
            else:
                pathlengthsadjust = []
                for l in pathlengths:
                    if l == 0:
                        pathlengthsadjust.append(100000)
                    else:
                        pathlengthsadjust.append(l)
                minlength = min(pathlengthsadjust)
                self.path = paths[pathlengths.index(minlength)]
    
    def UpdateMotion(self):
        
        tsize = self.h_tiledlayers.tilesize
        mapw = self.h_tiledlayers.map_size[0]
        init_tile = self.h_tiledlayers.map_size[0]*(self.y/tsize)+(self.x/tsize)
        
        # check for achieved waypoint
        if len(self.path) > 0:
            xdest = tsize*(self.path[0]%mapw) + tsize/2
            ydest = tsize*(self.path[0]/mapw) + tsize/2
            if (xdest-self.x) == 0 and (ydest-self.y) == 0:
                tile_achieved = self.path.pop(0)
                if len(self.path) > 0: # still moving
                    # do check here for changed map occupancy
                    if self.h_tiledlayers.occlayer[self.path[0]]:
                        self.path = []
                        self.task = []
                    else:
                        xdest = tsize*(self.path[0]%mapw) + tsize/2
                        ydest = tsize*(self.path[0]/mapw) + tsize/2
                else: # just finished current path, check for tasks
                    if self.alive == False:
                        self.floataway = True
                    if not self.task == [] and self.alive == True:
                        switch_tile = self.task[0]
                        switch_id = self.task[1]
                        if (tile_achieved == switch_tile):
                            self.h_tiledlayers.switchcontrol.switches[switch_id].Trigger()
                            if self.h_tiledlayers.switchcontrol.switches[switch_id].type == 'door':
                                doortiles = []
                                for h in self.h_tiledlayers.switchcontrol.switches[switch_id].h_assoc:
                                    doortiles.extend(h.tilelist)
                                self.h_tiledlayers.vacuumcontroller.UpdateVacuum(doortiles)
                                self.h_tiledlayers.ReRenderAirLayer()
                            elif self.h_tiledlayers.switchcontrol.switches[switch_id].type == 'pump':
                                num_success = 0
                                num_failure = 0
                                for h in self.h_tiledlayers.switchcontrol.switches[switch_id].h_assoc:
                                    if h.state == True:
                                        val = 0
                                    else:
                                        val = 1
                                    success = self.h_tiledlayers.vacuumcontroller.PumpAir(h.tile2,val,h.maxvolume)
                                    if success == True:
                                        self.h_tiledlayers.ReRenderAirLayer()
                                        num_success += 1
                                    else:
                                        num_failure += 1
                                if num_success > 0 and num_failure > 0:
                                    self.h_msgdialogue.DisplayMsg("pumppartialsuccess")
                                elif num_success > 0:
                                    if val == 0:
                                        self.h_msgdialogue.DisplayMsg("pumpdepressure")
                                    else:
                                        self.h_msgdialogue.DisplayMsg("pumprepressure")
                                else:
                                    self.h_msgdialogue.DisplayMsg("pumpfailure")
                            elif self.h_tiledlayers.switchcontrol.switches[switch_id].type == 'powerplug':
                                for h in self.h_tiledlayers.switchcontrol.switches[switch_id].h_assoc:
                                    h.powerstate = 1
                                self.numpowerpacks -= 1
                                self.h_msgdialogue.DisplayMsg("powerplugactivate")
            if abs(xdest-self.x) > 0:
                vx = (xdest-self.x)/abs(xdest-self.x)
            else:
                vx = 0
            if abs(ydest-self.y) > 0:
                vy = (ydest-self.y)/abs(ydest-self.y)
            else:
                vy = 0
            if self.floataway == True:
                vx = self.prevvx
                vy = self.prevvy
            self.prevvx = vx
            self.prevvy = vy
        elif self.floataway == True: # floating away
            vx = self.prevvx
            vy = self.prevvy
        else: # no control
            vx = 0
            vy = 0
        self.x = self.x + self.speed*vx
        self.y = self.y + self.speed*vy
        
        # Set gait type
        if abs(vx) > 0 or abs(vy) > 0:
            if self.alive == True:
                newgaittype = "walk"
            else:
                newgaittype = "floating"
        else:
            if self.alive == True:
                newgaittype = "stand"
            else:
                newgaittype = "dead"
        if not self.gaittype == newgaittype:
            self.gait_to = 0
        self.gaittype = newgaittype
        
        # Check if need to inform tilemap object layer of updates
        if self.alive == True:
            final_tile = self.h_tiledlayers.map_size[0]*(self.y/tsize)+(self.x/tsize)
            if not init_tile == final_tile and final_tile >=0 and \
                final_tile < self.h_tiledlayers.map_size[0]*self.h_tiledlayers.map_size[1]:
                self.h_tiledlayers.UpdateObj(init_tile,final_tile,self.id)
        
        # check for mentor messages
        if self.h_tiledlayers.level_id == "level_tutorial":
            if self.alive == True:
                m_inds = self.h_tiledlayers.FindObjects(final_tile,'mentor')
                if len(m_inds) > 0:
                    self.h_tiledlayers.RemoveObj(final_tile,m_inds[0]+800)
                    self.h_msgdialogue.DisplayMsg('none',mentor=True,mentor_ind=m_inds[0])
                    self.path = []
                    self.task = []
        
        # check for picking up power pack
        if self.alive == True:
            powerpack_inds = self.h_tiledlayers.FindObjects(final_tile,'key')
            if len(powerpack_inds) > 0:
                self.h_tiledlayers.RemoveObj(final_tile,powerpack_inds[0]+700)
                self.h_tiledlayers.powerpackcontrol.powerpacktiles[powerpack_inds[0]] = -1
                self.numpowerpacks += 1
                self.h_msgdialogue.DisplayMsg("pickupcell")
        
        # check if reached escape pod
        if self.alive == True:
            goal_inds = self.h_tiledlayers.FindObjects(final_tile,'goal')
            if len(goal_inds) > 0:
                self.escapepod = True
            else:
                self.escapepod = False
        
        # Set heading and gait info
        if vx > 0:
            self.heading = 'right'
        elif vx < 0:
            self.heading = 'left'
        
        # Player animation
        if self.gait_to == 0:
            if self.gaittype == 'stand':
                self.gait_to = 10
                self.gait = (self.gait + 1) % len(resources.player_spriteseq[self.type]['stand'])
            elif self.gaittype == 'walk':
                self.gait_to = 10
                self.gait = (self.gait + 1) % len(resources.player_spriteseq[self.type]['walk'])
            elif self.gaittype == 'floating':
                self.gait_to = 5
                self.gait = (self.gait + 1) % len(resources.player_spriteseq['unit']['floating'])
            else:
                self.gait = 0
                self.gait_to = 0
        elif self.gaittype == 'dead':
            self.gait = 0
            self.gait_to = 0
        self.gait_to = max(0,self.gait_to-1)
        #print self.gaittype, self.gait
        
        # check for death
        if self.alive == True:
            fire_inds = self.h_tiledlayers.FindObjects(final_tile,'fire')
            if len(fire_inds) > 0:
                self.Killed('fire')
                resources.soundfx['burn'].play()
            elif self.h_tiledlayers.airlayer[final_tile] == 0 and self.type == 'unit':
                self.Killed('vacuum')
                resources.soundfx['vacalarm'].play()
    
    def DrawPath(self, screen):
        tsize = self.h_tiledlayers.tilesize
        mapw = self.h_tiledlayers.map_size[0]
        for p in self.path[1:]:
            x = tsize*(p%mapw) + 9 - self.h_camera.x + self.h_camera.w_view/2
            y = tsize*(p/mapw) + 9 - self.h_camera.y + self.h_camera.h_view/2
            pygame.draw.rect(screen,(255,255,255),((x,y),(6,6)))
    
    def Draw(self,screen,drawtoescapepod=False):
        if self.gaittype == 'dead':
            return
        if drawtoescapepod == True and self.escapepod == True:
            ebx = self.h_tiledlayers.escapepod.relx
            xrender = self.x+ebx
        else:
            xrender = self.x
        if self.gaittype == 'floating' and self.x > -1000 and self.x < 1000 and self.y > -1000 and self.y < 1000:
            tilecoords = resources.player_spritesheet_coords[resources.player_spriteseq['unit']['floating'][self.gait]+self.charid]
            renderfrom = resources.player_spritesheet['floating']
        elif self.heading == 'right':
            tilecoords = resources.player_spritesheet_coords[resources.player_spriteseq[self.type][self.gaittype][self.gait]+self.charid]
            renderfrom = resources.player_spritesheet['right']
        else:
            tilecoords = resources.player_spritesheet_coords[resources.player_spriteseq[self.type][self.gaittype][self.gait]+self.charid]
            renderfrom = resources.player_spritesheet['left']
        
        screen.blit(renderfrom, (xrender-(self.tilesize/2)-self.h_camera.x+self.h_camera.w_view/2,self.y-(self.tilesize/2)-self.h_camera.y+self.h_camera.h_view/2 - 3), area=tilecoords)
        
        if self.alive == True:
            self.DrawPath(screen)

